/*
* This code is GPL.
*/
#include <netinet/in.h>
#include <linux/netfilter.h>

extern "C" {
#include <libipq.h>
}

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>

#include "hexdump.h"
#include "packet_processing.h"
#include "irUtil.h"

#define BUFSIZE 2048

/*
 * This programm uses libipq to receive packets from iptables/kernel space
 * and processes them.
 */

static void die(struct ipq_handle *h)
{
  ipq_perror("passer");
  ipq_destroy_handle(h);
  exit(1);
}

int main(int argc, char **argv)
{
  openlog(argv[0], LOG_PID, LOG_USER);

  int status;
  unsigned char buf[BUFSIZE];
  struct ipq_handle *h;

  pm_init();
  daemon(0,0);
  //iptables rule for apple mdns requests
  system("iptables -I FORWARD -p UDP -d 224.0.0.251 --destination-port 5353 --source-port 5353 -j QUEUE");

  syslog(LOG_USER|LOG_INFO, "Starting up...\n");

  h = ipq_create_handle(0, PF_INET);
  if (!h)
    die(h);

  status = ipq_set_mode(h, IPQ_COPY_PACKET, BUFSIZE);
  if (status < 0)
    die(h);

  do {
    status = ipq_read(h, buf, BUFSIZE, 0);
    if (status < 0)
      die(h);


    switch (ipq_message_type(buf)) {
    case NLMSG_ERROR:
      syslog(LOG_USER|LOG_WARNING, "Received error message %d\n",
              ipq_get_msgerr(buf));
      break;

    case IPQM_PACKET: {


      ipq_packet_msg_t *m = ipq_get_packet(buf);
      packet_t p;
      p.len = m->data_len;
      p.data = m->payload;
      readPacket(p);

      status = ipq_set_verdict(h, m->packet_id,
                               NF_ACCEPT, p.len, p.data);
      if (status < 0)
        die(h);
      break;
    }

    default:
      syslog(LOG_USER|LOG_ERR, "Unknown message type!\n");
      break;
    }
  } while (1);

  ipq_destroy_handle(h);
  return 0;
}
