#include <stdio.h>
#include <syslog.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>

#include "irUtil.h"
#include "irDaemon.h"

static EN_state_t g_state;

int main(int argc, char* argv[])
{
  daemon(0,0);
  g_state = EIS_init;
  openlog(argv[0], LOG_PID, LOG_USER);
  ir_init();
  syslog(LOG_USER|LOG_INFO, "Starting...\n");

  if(mkfifo(g_ird_ctrlFifo, 0666) < 0) {
    syslog(LOG_USER | LOG_ERR, "Failed to create fifo (%s) reason (%s)\n", g_ird_ctrlFifo, strerror(errno));
  }
  if(mkfifo(g_ird_infoFifo, 0666) < 0) {
    syslog(LOG_USER | LOG_ERR, "Failed to create fifo (%s) reason (%s)\n", g_ird_infoFifo, strerror(errno));
  }

  int fdCtrl = open(g_ird_ctrlFifo, O_RDWR);
  if(fdCtrl < 0) {
    syslog(LOG_USER | LOG_ERR, "Failed to open fifo (%s) reason (%s)\n", g_ird_ctrlFifo, strerror(errno));
  }

  int fdInfo = open(g_ird_infoFifo, O_RDWR);
  if(fdCtrl < 0) {
    syslog(LOG_USER | LOG_ERR, "Failed to open fifo (%s) reason (%s)\n", g_ird_infoFifo, strerror(errno));
  }

  while(1) {
    uint32_t msg = 0;
    int readRv = read(fdCtrl, &msg, IRD_MSG_LEN);
    if(readRv != IRD_MSG_LEN) {
      syslog(LOG_USER | LOG_ERR, "Failed to read message from ctrlFifo reason (%s)\n", strerror(errno));
    }

    switch(msg) {
      case IRD_MSG_SwitchToAppleTv:
        if(g_state != EIS_appleTv)  {
          ir_sendSwitchCommand(true);
          syslog(LOG_USER|LOG_INFO, "Switch now to appletv...\n");
        }
        g_state = EIS_appleTv;
        break;
      case IRD_MSG_SwitchToSat:
        if(g_state != EIS_sat) {
          ir_sendSwitchCommand(false);
          syslog(LOG_USER|LOG_INFO, "Switch now to sat...\n");
        }
        g_state = EIS_sat;
        break;
      case IRD_MSG_SingleSourceSwitch:
        ir_sendSwitchCommand(true);
        g_state = EIS_unknown;
        syslog(LOG_USER|LOG_INFO, "Switch now one single source click...\n");
        break;
      case IRD_MSG_GetStatus:
        if(write(fdInfo, &g_state, sizeof(EN_state_t)) < 0) {
          syslog(LOG_USER|LOG_ERR, "Failed to write to info fifo reason (%s)\n", strerror(errno));
        }
        syslog(LOG_USER|LOG_INFO, "Sending status...");
        break;
    }
  }
}
