#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <arpa/inet.h>

#include <stdint.h>

#include <linux/udp.h>
#include <linux/ip.h>
#include <string.h>
#include <syslog.h>
#include <errno.h>
#include "hexdump.h"

#include "packet_processing.h"

#include "irDaemon.h"

int findString(const char* str, uint8_t* data, size_t len);

/**Run only once!!!
 *
 */
void readMdnsHeader(uint8_t* src, struct mdnshdr** dest)
{
  *dest = (struct mdnshdr*)src;
  struct mdnshdr* hdr = *dest;
  hdr->id = ntohs(hdr->id);
  hdr->flags = ntohs(hdr->flags);
  hdr->qdcount = ntohs(hdr->qdcount);
  hdr->ancount = ntohs(hdr->ancount);
  hdr->nscount = ntohs(hdr->nscount);
  hdr->arcount = ntohs(hdr->arcount);
}

static int g_fdCtrl = 0;

void pm_init(void)
{
  g_fdCtrl = open(g_ird_ctrlFifo, O_RDWR);
  if(g_fdCtrl < 0) {
    syslog(LOG_USER|LOG_ERR, "Failed to open ctrl fifo reason (%s)", strerror(errno));
  }
}

void readPacket(packet_t p)
{
  uint8_t* dataPtr = p.data;

  struct iphdr* ipheader = (struct iphdr*)dataPtr;
  dataPtr += sizeof(struct iphdr);

  uint8_t* ipaddr = (uint8_t*)&(ipheader->saddr);

  struct udphdr* udpheader = (struct udphdr*)dataPtr;
  dataPtr += sizeof(struct udphdr);


  struct mdnshdr* mdnsheader;
  readMdnsHeader(dataPtr, &mdnsheader);
  dataPtr += sizeof(struct mdnshdr);

  uint8_t* mdnsPayload = dataPtr;
  size_t mdnsPayloadLen = p.len - (dataPtr - p.data);

  //hexdump(mdnsPayload, mdnsPayloadLen);

  /* Check if playback via AppleTV in enabled simply by finding the strings
   * inside the airplay protocol:
   * if iTunes_Ctrl_ and -iPad is present inside the same packet, assume playback is enabled.
   * if only iTunes_Ctrl_ is present, assume playback is disabled.
   */
  int iTunesCtrl = findString("iTunes_Ctrl_", mdnsPayload, mdnsPayloadLen);
  int iPad = findString("-iPad", mdnsPayload, mdnsPayloadLen);
  IRD_MSG_TYPE msg = 0;
  if (iTunesCtrl > 0 && iPad > 0) {
    msg = IRD_MSG_SwitchToAppleTv;
    if(write(g_fdCtrl, &msg, IRD_MSG_LEN) < 0) {
      syslog(LOG_USER|LOG_ERR, "Failed to write to ctrl fifo reason (%s)\n", strerror(errno));
    }
  } else if(iTunesCtrl > 0) {
    msg = IRD_MSG_SwitchToSat;
    if(write(g_fdCtrl, &msg, IRD_MSG_LEN) < 0) {
      syslog(LOG_USER|LOG_ERR, "Failed to write to ctrl fifo reason (%s)\n", strerror(errno));
    }
  }
}

int findString(const char* str, uint8_t* data, size_t len)
{
  int rv = -1;
  size_t off=0;
  while(off < len) {
    int i;
    for(i=0; i<strlen(str) && off < len; i++)
    {
      if(*(data+off) != str[i]) {
        break;
      }
      off++;
    }
    if(i == strlen(str)) {
      rv = off - strlen(str);
      break;
    }
    off++;
  }

  return rv;  
}

